/* Copyright (c) 1999-2001 by Juliusz Chroboczek.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE. */


#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <X11/Shell.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Viewport.h>
#include <X11/Xaw/List.h>
#include <X11/Xaw/Scrollbar.h>
#include <stdio.h>
#include "UGrid.h"

char *ProgramName;
static XrmOptionDescRec ucm_options[] = {
{"-fn",		"*grid.font",	XrmoptionSepArg,	(caddr_t) NULL },
};

static void Quit(Widget, XEvent*, String*, Cardinal*);
static void Prev(Widget, XEvent*, String*, Cardinal*);
static void Next(Widget, XEvent*, String*, Cardinal*);
static void Clear(Widget, XEvent*, String*, Cardinal*);
static void set_start_state(Dimension);
static void set_selected_state(Dimension);
static void ListCallback(Widget, XtPointer, XtPointer);
static void GridCallback(Widget, XtPointer, XtPointer);

static XtActionsRec ucm_actions[] = {
    { "Quit", Quit },
    { "Prev", Prev },
    { "Next", Next },
    { "Clear", Clear },
};

static XtAppContext xtcontext;
static Widget 
  toplevel, 
    paned,
      form, 
        box,
          nextButton, prevButton, quitButton, clearButton,
        label,
      vpaned,
        ugrid,
        viewport,
          list;

static Atom wm_delete_window;

int
main(int argc, char **argv) 
{
    Arg arglist[4];
    int num;
    String blockNames[0x101];
    int i;
    static XtCallbackRec lcb[2] = {{ListCallback, NULL}, {NULL, NULL}};
    static XtCallbackRec gcb[2] = {{GridCallback, NULL}, {NULL, NULL}};
    
    toplevel = XtAppInitialize (&xtcontext, "Ucm",
                                ucm_options, XtNumber(ucm_options),
                                &argc, argv, NULL, NULL, 0);
    
    if (argc != 1) {
        fprintf(stderr, "Usage: %s [options]\n", argv[0]);
        exit(2);
    }

    XtAppAddActions (xtcontext, ucm_actions, XtNumber(ucm_actions));
    XtOverrideTranslations
        (toplevel,
         XtParseTranslationTable("<Message>WM_PROTOCOLS: Quit()"));

    paned = XtCreateManagedWidget("paned", panedWidgetClass, toplevel, 
                                  NULL, ZERO);
    num = 0;
    form = 
      XtCreateManagedWidget("form", formWidgetClass, paned, arglist, num);

    num = 0;
    XtSetArg(arglist[num], XtNorientation, XtorientHorizontal); num++;
    box = 
      XtCreateManagedWidget("box", boxWidgetClass, form, arglist, num);

    quitButton = XtCreateManagedWidget ("quit", commandWidgetClass, box,
                                        NULL, ZERO);
    prevButton = XtCreateManagedWidget ("prev", commandWidgetClass, box,
                                        NULL, ZERO);
    nextButton = XtCreateManagedWidget ("next", commandWidgetClass, box,
                                        NULL, ZERO);
    clearButton = XtCreateManagedWidget ("clear", commandWidgetClass, box,
                                         NULL, ZERO);
    XtAugmentTranslations
        (quitButton,
         XtParseTranslationTable("<Btn1Down>,<Btn1Up>: Quit() unset()"));
    XtAugmentTranslations
        (prevButton,
         XtParseTranslationTable("<Btn1Down>,<Btn1Up>: Prev() unset()"));
    XtAugmentTranslations
        (nextButton,
         XtParseTranslationTable("<Btn1Down>,<Btn1Up>: Next() unset()"));
    XtAugmentTranslations
        (clearButton,
         XtParseTranslationTable("<Btn1Down>,<Btn1Up>: Clear() unset()"));

    num = 0;
    XtSetArg(arglist[num], XtNfromHoriz, box); num++;
    XtSetArg(arglist[num], XtNlabel, "         "); num++;
    label = XtCreateManagedWidget("label", labelWidgetClass, form,
                                  arglist, num);

    num = 0;
    XtSetArg(arglist[num], XtNorientation, XtorientHorizontal); num++;
    vpaned = XtCreateManagedWidget("vpaned", panedWidgetClass, paned, 
                                   arglist, num);
    
    num = 0;
    XtSetArg(arglist[num], XtNcallback, gcb); num++;
    ugrid = XtCreateManagedWidget("grid", ugridWidgetClass, vpaned,
                                  arglist, num);

    num = 0;
    XtSetArg(arglist[num], XtNallowVert, True); num++;
    XtSetArg(arglist[num], XtNforceBars, True); num++;
    XtSetArg(arglist[num], XtNskipAdjust, True); num++;
    viewport = XtCreateManagedWidget("viewport", viewportWidgetClass, vpaned,
                                     arglist, num);

    for(i = 0; i < 0x100; i++) {
        blockNames[i]=XtMalloc(9);
        sprintf(blockNames[i], "U+%02X00  ", i);
    }
    blockNames[0x100]=NULL;

    num = 0;
    XtSetArg(arglist[num], XtNlist, blockNames); num++;
    XtSetArg(arglist[num], XtNcallback, lcb); num++;
    XtSetArg(arglist[num], XtNforceColumns, True); num++;
    XtSetArg(arglist[num], XtNdefaultColumns, 1); num++;
    list = XtCreateManagedWidget("list", listWidgetClass, viewport,
                                 arglist, num);
    XawListHighlight(list, 0);
    
    num = 0;
    XtSetArg(arglist[num], XtNheight, 100); num++;
    XtSetValues(viewport, arglist, num);
    
    set_start_state(0);
    set_selected_state(NO_CODEPOINT);

    XtRealizeWidget(toplevel);
    wm_delete_window = XInternAtom(XtDisplay(toplevel), "WM_DELETE_WINDOW",
                                   False);
    XSetWMProtocols(XtDisplay(toplevel), XtWindow(toplevel),
                    &wm_delete_window, 1);
    XtAppMainLoop(xtcontext);

    return 0;
}

static void
Quit(Widget w, XEvent *event, String *params, Cardinal *num_params)
{
    if(event->type == ClientMessage && 
       event->xclient.data.l[0] != wm_delete_window) {
        XBell(XtDisplay(w), 0);
    } else {
        XtDestroyApplicationContext(xtcontext);
        exit(0);
    }
}

static void
Next(Widget w, XEvent *event, String *params, Cardinal *num_params)
{
    XtCallActionProc(ugrid, "PageForward", event, NULL, ZERO);
}

static void
Prev(Widget w, XEvent *event, String *params, Cardinal *num_params)
{
    XtCallActionProc(ugrid, "PageBackward", event, NULL, ZERO);
}

static void
Clear(Widget w, XEvent *event, String *params, Cardinal *num_params)
{
    XtCallActionProc(ugrid, "ClearSelection", event, NULL, ZERO);
}

static void 
set_start_state(Dimension start_codepoint)
{
    Arg arglist[1];
    int num;

    num = 0;
    XtSetArg(arglist[num], XtNsensitive,
             start_codepoint>0?True:False); num++;
    XtSetValues(prevButton, arglist, num);

    num = 0;
    XtSetArg(arglist[num], XtNsensitive,
             start_codepoint < 0xFF00 ? True : False); num++;
    XtSetValues(nextButton, arglist, num);
}

static void 
set_selected_state(Dimension selected_codepoint)
{
    Arg arglist[1];
    int num;
    static char l[10];

    num = 0;
    XtSetArg(arglist[0], XtNsensitive,
             CODEPOINT(selected_codepoint)?True:False); num++;
    XtSetValues(clearButton, arglist, num);

    if(CODEPOINT(selected_codepoint))
        sprintf(l, "U+%04X", selected_codepoint);
    else
        strcpy(l, "         ");
    num = 0;
    XtSetArg(arglist[0], XtNlabel, l); num++;
    XtSetValues(label, arglist, num);
}

static void
ListCallback(Widget w, XtPointer closure, XtPointer data)
{
    XawListReturnStruct *lr = data;
    Arg arglist[1];
    int num;

    if(lr->list_index < 0 || lr->list_index >= 0x100)
        return;

    num = 0;
    XtSetArg(arglist[num], XtNstartCodepoint, 
             (Dimension)(lr->list_index * 0x100l)); num++;
    XtSetValues(ugrid, arglist, num);

    set_start_state(lr->list_index * 0x100l);
}

static void
GridCallback(Widget w, XtPointer closure, XtPointer data)
{
    UGridCallbackDataRec *rec = data;
    int n;

    XawListUnhighlight(list);
    n = rec->start_codepoint / 0x100l;
    if(n >= 0 && n <= 0x100) {
        XawListHighlight(list, n);
    }
    set_start_state(rec->start_codepoint);
    set_selected_state(rec->selected_codepoint);
}
